-- Bomb
-- RJ 20200819

function createBombTypes()
    BombTypes = {
{readImage(asset.plunger1), readImage(asset.plunger2), readImage(asset.plunger3), readImage(asset.plunger4)},
{readImage(asset.rolling1), readImage(asset.rolling2), readImage(asset.rolling3), readImage(asset.rolling4)},
{readImage(asset.squig1), readImage(asset.squig2), readImage(asset.squig3), readImage(asset.squig4)}
    }
    BombExplosion = readImage(asset.alien_shot_exploding)
end


Bomb = class()

function Bomb:init(pos)
    self.pos = pos
    self.shapes = BombTypes[math.random(3)]
    self.shape = 0
    self.explodeCount = 0
end

function Bomb:draw()
    if self.explodeCount > 0 then
        sprite(BombExplosion, self.pos.x, self.pos.y)
        self.explodeCount = self.explodeCount - 1
        if self.explodeCount == 0 then self.army:deleteBomb(self) end
    else
        sprite(self.shapes[self.shape + 1],self.pos.x,self.pos.y)
    end
end

function Bomb:update(army, increment)
    if self.explodeCount > 0 then return end
    self.pos.y = self.pos.y - (increment or 1)
    self.shape = (self.shape + 1)%4
    self:checkCollisions(army)
end

function Bomb:checkCollisions(army)
    if self:killedShield() then
        -- intentionally blank
        -- would be better to move shield damaging up here.
    elseif self:killsGunner() then
        Gunner:explode()
    elseif self.pos.y < 16 then
        self:damageLine(self)
    else
        return -- without exploding
    end
    self:explode(army)
end

function Bomb:explode(army)
    self.pos = self.pos - vec2(2,3)
    self.army = army
    self.explodeCount = 15
end

function Bomb:killedShield()
    for i,shield in ipairs(Shields) do
        local hit = self:damageShield(shield)
        if hit then return true end
    end
    return false
end

function Bomb:damageShield(shield)
    local shieldPos = shield.pos
    local hitPos = rectanglesIntersectAt(self.pos,3,4, shieldPos,22,16)
    if hitPos == nil then return false end
    if not shield:hitBits(self.pos,3,4) then return false end
    shield:applyDamage(hitPos)
    return true
end

function Bomb:damageLine(bomb)
    local tx = bomb.pos.x - 7
    for x = 1,6 do
        if math.random() < 0.5 then
            Line:set(tx+x,1, 0,0,0,0)
        end
    end            
end

function rectanglesIntersectAt(bl1,w1,h1, bl2,w2,h2)
    if rectanglesIntersect(bl1,w1,h1, bl2,w2,h2) then
        return bl1
    else
        return nil
    end
end

function rectanglesIntersect(bl1,w1,h1, bl2,w2,h2)
   local tr1 = bl1 + vec2(w1,h1) - vec2(1,1)
   local tr2 = bl2 + vec2(w2,h2) - vec2(1,1)
   if bl1.y > tr2.y or bl2.y > tr1.y then return false end
   if bl1.x > tr2.x or bl2.x > tr1.x then return false end
   return true
end

function Bomb:killsGunner()
    if not Gunner.alive then return false end
    local hit = rectanglesIntersectAt(self.pos,3,4, Gunner.pos,16,8)
    if hit == nil then return false end
    return true
end
