-- RJ 20200803
    
function testCodeaUnitFunctionality()
    CodeaUnit.detailed = true

    _:describe("Invaders Test Suite", function()

        _:before(function()
            createBombTypes()
        end)

        _:after(function()
            -- Some teardown
        end)

        _:test("Hookup test", function()
            _:expect("Foo").is("Foo")
        end)
        
        _:test("Screen is 1366 wide", function()
            _:expect(WIDTH).is(1366)
        end)
        
        _:test("Screen facts", function()
            local gameConsumes = 244*4
            _:expect(gameConsumes).is(976)
            local leftMargin = 1366/2 - 244*2
            _:expect(leftMargin).is(195)
            local fireTouch = 1366-195
            _:expect(fireTouch).is(1171)
            local moveLeft = 195//2
            _:expect(moveLeft).is(97)
        end)
        
        _:test("invaderNumber starts at 1", function()
            local army = Army()
            _:expect(army.invaderNumber).is(1)
        end)
        
        _:test("invaderNumber increments", function()
            local army = Army()
            _:expect(army.invaderNumber).is(1)
            army:nextInvader()
            _:expect(army.invaderNumber).is(2)
        end)
        
        _:test("nextInvader returns an instance", function()
            local army = Army()
            local i1 = army:nextInvader()
            _:expect(i1).isnt(nil)
        end)
        
        _:test("nextInvader returns unique instances", function()
            local army = Army()
            local i1 = army:nextInvader()
            local i2 = army:nextInvader()
            _:expect(i1).isnt(i2)
        end)
        
        _:test("there are invaders", function()
            local army = Army()
            local n = #army.invaders
            _:expect(n).is(55)
            local inv = #army.invaders[1]
            _:expect(inv).isnt(nil)
        end)
        
        _:test("over right edge", function()
            local army = Army()
            local testInvader = army.invaders[1]
            army.invaders = {testInvader}
            testInvader.pos.x = 203
            army:doUpdate()
            _:expect(army.overTheEdge).is(false)
            _:expect(testInvader.pos.x).is(205)
            army:doUpdate()
            _:expect(testInvader.pos.x).is(207)
            _:expect(army.overTheEdge).is(true)
        end)
        
        _:test("bombs get deleted", function()
            Shields = {}
            Gunner = {pos=vec2(112,10)-vec2(8,4),alive=true,count=0,explode=explode}
            local army = Army()
            local bomb = Bomb(vec2(122,10))
            army:dropBomb(bomb)
            _:expect(countTable(army.bombs)).is(1)
            for i = 1,10 do
                bomb:update(army,1)
                _:expect(countTable(army.bombs)).is(1)
            end
            bomb:update(army,1)
            _:expect(countTable(army.bombs)).is(0)
        end)
        
        _:test("Bomb hits gunner", function()
            Gunner = {pos=vec2(50,50)}
            -- gunner's rectangle is 16x8 on CORNER
            -- covers x = 50-65 y = 50,57
            local bomb = Bomb(vec2(50,50))
            -- bomb is 3x4, covers x = 50-52 y = 50-53
            _:expect(bomb:killsGunner()).is(true)
            bomb.pos.y = 58
            _:expect(bomb:killsGunner()).is(false)
            bomb.pos.y = 57
            _:expect(bomb:killsGunner()).is(true)
            bomb.pos.x = 48 -- covers x = 48,49,50
            _:expect(bomb:killsGunner()).is(true)
            bomb.pos.x = 47 -- 47,48,49
            _:expect(bomb:killsGunner()).is(false)
            bomb.pos.x = 65 -- 65,66,67
            _:expect(bomb:killsGunner()).is(true)
            bomb.pos.x = 66 -- 66,67,68
            _:expect(bomb:killsGunner()).is(false)
        end)
        
        _:ignore("Bomb damages shield", function()
            local hitAt
            local bomb = Bomb(vec2(100,100))
            local original = bomb.applyDamage
            local apply = function(object, shield, pos)
                hitAt = pos
                return original(object, shield, pos)
            end
            bomb.applyDamage = apply
            local shield = Shield(readImage(asset.shield), vec2(89,92))
            bomb:damageShield(shield)
            _:expect(hitAt).is(vec2(100,100))
        end)
        
        _:test("rectangle high right hits no visible bits", function()
            local shield = testShield()
            local hitNo = shield:hitBits(vec2(112,108),3,4)
            _:expect(hitNo).is(false)
        end)
        
        _:test("rectangle at origin hits visible bits", function()
            local shield = testShield()
            local hitYes = shield:hitBits(vec2(100,100),3,4)
            _:expect(hitYes).is(true)
        end)
        
        _:test("rectangle off to left misses visible bits", function()
            local shield = testShield()
            local hit = shield:hitBits(vec2(97,100),3,4)
            _:expect(hit).is(false)
        end)
        
        _:test("rectangle just above hits visible bits", function()
            local shield = testShield()
            local hit = shield:hitBits(vec2(105,107),3,4)
            _:expect(hit).is(true)
        end)

    end)
end

function testShield()
    local white = color(255)
    local img = image(22,16) -- lower left quadrant white
    for x=1,11 do
        for y = 1,8 do
            img:set(x,y,white)
        end
    end
    return Shield(img,vec2(100,100))
end

function countTable(t)
    local c = 0
    for k,v in pairs(t) do
        c = c + 1
    end
    return c
end
